@@ -1,29 +1,28 @@
 from __future__ import absolute_import, division, print_function
 
-from abc import ABCMeta
 from collections import OrderedDict, Iterator
 from functools import partial
 from hashlib import md5
+from operator import getitem
 import inspect
 import pickle
 import os
 import threading
 import uuid
-import warnings
 
 from toolz import merge, groupby, curry, identity
 from toolz.functoolz import Compose
 
 from .compatibility import long, unicode
 from .context import _globals, thread_state
-from .core import flatten
+from .core import flatten, quote, get as simple_get
 from .hashing import hash_buffer_hex
 from .utils import Dispatch, ensure_dict
 
 
 __all__ = ("DaskMethodsMixin",
            "is_dask_collection",
-           "compute", "persist", "visualize",
+           "compute", "persist", "optimize", "visualize",
            "tokenize", "normalize_token")
 
 
@@ -125,7 +124,7 @@ class DaskMethodsMixin(object):
         --------
         dask.base.persist
         """
-        (result,) = persist(self, **kwargs)
+        (result,) = persist(self, traverse=False, **kwargs)
         return result
 
     def compute(self, **kwargs):
@@ -156,73 +155,6 @@ class DaskMethodsMixin(object):
         return result
 
 
-def call_finalize(finalize, args, results):
-    return finalize(results, *args)
-
-
-def add_ABCMeta(cls):
-    """Use the metaclass ABCMeta for this class"""
-    return ABCMeta(cls.__name__, cls.__bases__, cls.__dict__.copy())
-
-
-# TODO: this class is deprecated and should be removed in a future release.
-@add_ABCMeta
-class Base(DaskMethodsMixin):
-    """DEPRECATED. The recommended way to create a custom dask object now is to
-    implement the dask collection interface (see the docs), and optionally
-    subclass from ``DaskMethodsMixin`` if desired.
-
-    See http://dask.pydata.org/en/latest/custom-collections.html for more
-    information"""
-    __slots__ = ()
-
-    @classmethod
-    def __subclasshook__(cls, other):
-        if cls is Base:
-            warnings.warn("DeprecationWarning: `dask.base.Base` is deprecated. "
-                          "To check if an object is a dask collection use "
-                          "dask.base.is_dask_collection.\n\nSee http://dask."
-                          "pydata.org/en/latest/custom-collections.html "
-                          " for more information")
-        return NotImplemented
-
-    def __dask_graph__(self):
-        # We issue a deprecation warning for the whole class here, as any
-        # non-instance check usage will end up calling `__dask_graph__`.
-        warnings.warn("DeprecationWarning: `dask.base.Base` is deprecated. "
-                      "To create a custom dask object implement the dask "
-                      "collection interface, and optionally subclass from "
-                      "``DaskMethodsMixin`` if desired.\n\nSee http://dask."
-                      "pydata.org/en/latest/custom-collections.html "
-                      " for more information")
-        return self.dask
-
-    def _keys(self):
-        warnings.warn("DeprecationWarning: the `_keys` method is deprecated, "
-                      "use `__dask_keys__` instead")
-        return self.__dask_keys__()
-
-    @property
-    def _finalize(self):
-        warnings.warn("DeprecationWarning: the `_finalize` method is "
-                      "deprecated, use `__dask_postcompute__` instead")
-        f, args = self.__dask_postcompute__()
-        return partial(call_finalize, f, args) if args else f
-
-    @classmethod
-    def _optimize(cls, *args, **kwargs):
-        warnings.warn("DeprecationWarning: the `_optimize` method is "
-                      "deprecated, use `__dask_optimize__` instead")
-        return cls.__dask_optimize__(*args, **kwargs)
-
-    @classmethod
-    def _get(cls, dsk, keys, **kwargs):
-        warnings.warn("DeprecationWarning: the `_get` method is "
-                      "deprecated, use ``dask.base.compute_as_if_collection`` "
-                      "instead")
-        return compute_as_if_collection(cls, dsk, keys, **kwargs)
-
-
 def compute_as_if_collection(cls, dsk, keys, get=None, **kwargs):
     """Compute a graph as if it were of type cls.
 
@@ -281,6 +213,74 @@ def _extract_graph_and_keys(vals):
     return dsk, keys
 
 
+def unpack_collections(*args, **kwargs):
+    """Extract collections in preparation for compute/persist/etc...
+
+    Intended use is to find all collections in a set of (possibly nested)
+    python objects, do something to them (compute, etc...), then repackage them
+    in equivalent python objects.
+
+    Parameters
+    ----------
+    *args
+        Any number of objects. If it is a dask collection, it's extracted and
+        added to the list of collections returned. By default, python builtin
+        collections are also traversed to look for dask collections (for more
+        information see the ``traverse`` keyword).
+    traverse : bool, optional
+        If True (default), builtin python collections are traversed looking for
+        any dask collections they might contain.
+
+    Returns
+    -------
+    collections : list
+        A list of all dask collections contained in ``args``
+    repack : callable
+        A function to call on the transformed collections to repackage them as
+        they were in the original ``args``.
+    """
+    traverse = kwargs.pop('traverse', True)
+
+    collections = []
+    repack_dsk = {}
+
+    def _unpack(expr):
+        if is_dask_collection(expr):
+            tok = tokenize(expr)
+            if tok not in repack_dsk:
+                repack_dsk[tok] = (getitem, 'collections', len(collections))
+                collections.append(expr)
+            return tok
+
+        tok = uuid.uuid4().hex
+        if not traverse:
+            tsk = quote(expr)
+        else:
+            # Treat iterators like lists
+            typ = list if isinstance(expr, Iterator) else type(expr)
+
+            if typ in (list, tuple, set):
+                tsk = (typ, [_unpack(i) for i in expr])
+            elif typ is dict:
+                tsk = (dict, [[_unpack(k), _unpack(v)]
+                              for k, v in expr.items()])
+            else:
+                return expr
+
+        repack_dsk[tok] = tsk
+        return tok
+
+    out = uuid.uuid4().hex
+    repack_dsk[out] = (tuple, [_unpack(i) for i in args])
+
+    def repack(results):
+        dsk = repack_dsk.copy()
+        dsk['collections'] = quote(results)
+        return simple_get(dsk, out)
+
+    return collections, repack
+
+
 def optimize(*args, **kwargs):
     """Optimize several dask collections at once.
 
@@ -298,6 +298,11 @@ def optimize(*args, **kwargs):
         merged with all those of all other dask objects before returning an
         equivalent dask collection. Non-dask arguments are passed through
         unchanged.
+    traverse : bool, optional
+        By default dask traverses builtin python collections looking for dask
+        objects passed to ``optimize``. For large collections this can be
+        expensive. If none of the arguments contain any dask objects, set
+        ``traverse=False`` to avoid doing this traversal.
     optimizations : list of callables, optional
         Additional optimization passes to perform.
     **kwargs
@@ -315,15 +320,20 @@ def optimize(*args, **kwargs):
     >>> b2.compute() == b.compute()
     True
     """
-    variables = [a for a in args if is_dask_collection(a)]
-    if not variables:
+    collections, repack = unpack_collections(*args, **kwargs)
+    if not collections:
         return args
 
-    dsk = collections_to_dsk(variables, **kwargs)
+    dsk = collections_to_dsk(collections, **kwargs)
     postpersists = [a.__dask_postpersist__() if is_dask_collection(a)
                     else (None, a) for a in args]
 
-    return tuple(a if f is None else f(dsk, *a) for f, a in postpersists)
+    keys, postpersists = [], []
+    for a in collections:
+        keys.extend(flatten(a.__dask_keys__()))
+        postpersists.append(a.__dask_postpersist__())
+
+    return repack([r(dsk, *s) for r, s in postpersists])
 
 
 # TODO: remove after deprecation cycle of `dask.optimize` module completes
@@ -371,40 +381,31 @@ def compute(*args, **kwargs):
     >>> compute({'a': a, 'b': b, 'c': 1})  # doctest: +SKIP
     ({'a': 45, 'b': 4.5, 'c': 1},)
     """
-    from dask.delayed import delayed
     traverse = kwargs.pop('traverse', True)
-    if traverse:
-        args = tuple(delayed(a)
-                     if isinstance(a, (list, set, tuple, dict, Iterator))
-                     else a for a in args)
-
     optimize_graph = kwargs.pop('optimize_graph', True)
-    variables = [a for a in args if is_dask_collection(a)]
-    if not variables:
-        return args
-
     get = kwargs.pop('get', None) or _globals['get']
 
+    collections, repack = unpack_collections(*args, traverse=traverse)
+    if not collections:
+        return args
+
     if get is None and getattr(thread_state, 'key', False):
         from distributed.worker import get_worker
         get = get_worker().client.get
 
     if not get:
-        get = variables[0].__dask_scheduler__
-        if not all(a.__dask_scheduler__ == get for a in variables):
+        get = collections[0].__dask_scheduler__
+        if not all(a.__dask_scheduler__ == get for a in collections):
             raise ValueError("Compute called on multiple collections with "
                              "differing default schedulers. Please specify a "
                              "scheduler `get` function using either "
                              "the `get` kwarg or globally with `set_options`.")
 
-    dsk = collections_to_dsk(variables, optimize_graph, **kwargs)
-    keys = [var.__dask_keys__() for var in variables]
-    postcomputes = [a.__dask_postcompute__() if is_dask_collection(a)
-                    else (None, a) for a in args]
+    dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
+    keys = [x.__dask_keys__() for x in collections]
+    postcomputes = [x.__dask_postcompute__() for x in collections]
     results = get(dsk, keys, **kwargs)
-    results_iter = iter(results)
-    return tuple(a if f is None else f(next(results_iter), *a)
-                 for f, a in postcomputes)
+    return repack([f(r, *a) for r, (f, a) in zip(results, postcomputes)])
 
 
 def visualize(*args, **kwargs):
@@ -535,6 +536,11 @@ def persist(*args, **kwargs):
         A scheduler ``get`` function to use. If not provided, the default
         is to check the global settings first, and then fall back to
         the collection defaults.
+    traverse : bool, optional
+        By default dask traverses builtin python collections looking for dask
+        objects passed to ``persist``. For large collections this can be
+        expensive. If none of the arguments contain any dask objects, set
+        ``traverse=False`` to avoid doing this traversal.
     optimize_graph : bool, optional
         If True [default], the graph is optimized before computation.
         Otherwise the graph is run as is. This can be useful for debugging.
@@ -545,12 +551,14 @@ def persist(*args, **kwargs):
     -------
     New dask collections backed by in-memory data
     """
-    collections = [a for a in args if is_dask_collection(a)]
+    traverse = kwargs.pop('traverse', True)
+    optimize_graph = kwargs.pop('optimize_graph', True)
+    get = kwargs.pop('get', None) or _globals['get']
+
+    collections, repack = unpack_collections(*args, traverse=traverse)
     if not collections:
         return args
 
-    get = kwargs.pop('get', None) or _globals['get']
-
     if get is None and getattr(thread_state, 'key', False):
         from distributed.worker import get_worker
         get = get_worker().client.get
@@ -566,43 +574,32 @@ def persist(*args, **kwargs):
             except ValueError:
                 pass
             else:
-                if client.get == _globals['get']:
-                    collections = client.persist(collections, **kwargs)
-                    if isinstance(collections, list):  # distributed is inconsistent here
-                        collections = tuple(collections)
-                    else:
-                        collections = (collections,)
-                    results_iter = iter(collections)
-                    return tuple(a if not is_dask_collection(a)
-                                 else next(results_iter)
-                                 for a in args)
-
-    optimize_graph = kwargs.pop('optimize_graph', True)
+                if client.get == get:
+                    results = client.persist(collections,
+                                             optimize_graph=optimize_graph,
+                                             **kwargs)
+                    return repack(results)
 
     if not get:
         get = collections[0].__dask_scheduler__
         if not all(a.__dask_scheduler__ == get for a in collections):
-            raise ValueError("Compute called on multiple collections with "
+            raise ValueError("Persist called on multiple collections with "
                              "differing default schedulers. Please specify a "
                              "scheduler `get` function using either "
                              "the `get` kwarg or globally with `set_options`.")
 
     dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
-
     keys, postpersists = [], []
-    for a in args:
-        if is_dask_collection(a):
-            a_keys = list(flatten(a.__dask_keys__()))
-            rebuild, state = a.__dask_postpersist__()
-            keys.extend(a_keys)
-            postpersists.append((rebuild, a_keys, state))
-        else:
-            postpersists.append((None, None, a))
+    for a in collections:
+        a_keys = list(flatten(a.__dask_keys__()))
+        rebuild, state = a.__dask_postpersist__()
+        keys.extend(a_keys)
+        postpersists.append((rebuild, a_keys, state))
 
     results = get(dsk, keys, **kwargs)
     d = dict(zip(keys, results))
-    return tuple(s if r is None else r({k: d[k] for k in ks}, *s)
-                 for r, ks, s in postpersists)
+    results2 = [r({k: d[k] for k in ks}, *s) for r, ks, s in postpersists]
+    return repack(results2)
 
 
 ############
