@@ -2,6 +2,7 @@ import os
 import pytest
 import requests
 import subprocess
+import sys
 import time
 
 from dask.bytes.core import open_files
@@ -19,9 +20,9 @@ def dir_server():
                 f.write(b'a' * 10000)
 
         if PY2:
-            cmd = ['python', '-m', 'SimpleHTTPServer', '8999']
+            cmd = [sys.executable, '-m', 'SimpleHTTPServer', '8999']
         else:
-            cmd = ['python', '-m', 'http.server', '8999']
+            cmd = [sys.executable, '-m', 'http.server', '8999']
         p = subprocess.Popen(cmd, cwd=d)
         timeout = 10
         while True:
