@@ -4,7 +4,8 @@ import re
 import copy
 import json
 import warnings
-import distutils
+import os
+from distutils.version import LooseVersion
 
 import numpy as np
 import pandas as pd
@@ -12,12 +13,14 @@ import pandas as pd
 from ..core import DataFrame, Series
 from ..utils import (clear_known_categories, strip_unknown_categories,
                      UNKNOWN_CATEGORIES)
+from ...bytes.compression import compress
 from ...base import tokenize
 from ...compatibility import PY3, string_types
 from ...delayed import delayed
 from ...bytes.core import get_fs_token_paths
 from ...bytes.utils import infer_storage_options
-from ...utils import import_required
+from ...utils import import_required, natural_sort_key
+from .utils import _get_pyarrow_dtypes, _meta_from_dtypes
 
 __all__ = ('read_parquet', 'to_parquet')
 
@@ -100,50 +103,6 @@ def _parse_pandas_metadata(pandas_metadata):
     return index_names, column_names, storage_name_mapping, column_index_names
 
 
-def _meta_from_dtypes(to_read_columns, file_dtypes, index_cols,
-                      column_index_names):
-    """Get the final metadata for the dask.dataframe
-
-    Parameters
-    ----------
-    to_read_columns : list
-        All the columns to end up with, including index names
-    file_dtypes : dict
-        Mapping from column name to dtype for every element
-        of ``to_read_columns``
-    index_cols : list
-        Subset of ``to_read_columns`` that should move to the
-        index
-    column_index_names : list
-        The values for df.columns.name for a MultiIndex in the
-        columns, or df.index.name for a regular Index in the columns
-
-    Returns
-    -------
-    meta : DataFrame
-    """
-    meta = pd.DataFrame({c: pd.Series([], dtype=d)
-                         for (c, d) in file_dtypes.items()},
-                        columns=to_read_columns)
-    df = meta[list(to_read_columns)]
-
-    if not index_cols:
-        return df
-    if not isinstance(index_cols, list):
-        index_cols = [index_cols]
-    df = df.set_index(index_cols)
-    # XXX: this means we can't roundtrip dataframes where the index names
-    # is actually __index_level_0__
-    if len(index_cols) == 1 and index_cols[0] == '__index_level_0__':
-        df.index.name = None
-
-    if len(column_index_names) == 1:
-        df.columns.name = column_index_names[0]
-    else:
-        df.columns.names = column_index_names
-    return df
-
-
 def _normalize_index_columns(user_columns, data_columns, user_index, data_index):
     """Normalize user and file-provided column and index names
 
@@ -216,7 +175,7 @@ def _normalize_index_columns(user_columns, data_columns, user_index, data_index)
 
 
 def _read_fastparquet(fs, fs_token, paths, columns=None, filters=None,
-                      categories=None, index=None):
+                      categories=None, index=None, infer_divisions=None):
     import fastparquet
     from fastparquet.util import check_column_names
 
@@ -230,6 +189,11 @@ def _read_fastparquet(fs, fs_token, paths, columns=None, filters=None,
         except Exception:
             pf = fastparquet.ParquetFile(paths[0], open_with=fs.open, sep=fs.sep)
 
+    # Validate infer_divisions
+    if os.path.split(pf.fn)[-1] != '_metadata' and infer_divisions is True:
+        raise NotImplementedError("infer_divisions=True is not supported by the fastparquet engine for datasets "
+                                  "that do not contain a global '_metadata' file")
+
     check_column_names(pf.columns, categories)
     if isinstance(columns, tuple):
         # ensure they tokenize the same
@@ -318,7 +282,7 @@ def _read_fastparquet(fs, fs_token, paths, columns=None, filters=None,
         divisions = (None, None)
         return out_type(dsk, name, meta, divisions)
 
-    if index_names:
+    if index_names and infer_divisions is not False:
         index_name = meta.index.name
         minmax = fastparquet.api.sorted_partitioned_columns(pf)
         if index_name in minmax:
@@ -327,8 +291,17 @@ def _read_fastparquet(fs, fs_token, paths, columns=None, filters=None,
             divisions = [divisions[i] for i, rg in enumerate(pf.row_groups)
                          if rg in rgs] + [divisions[-1]]
         else:
+            if infer_divisions is True:
+                raise ValueError(
+                    ("Unable to infer divisions for index of '{index_name}' because it is not known to be "
+                     "sorted across partitions").format(index_name=index_name))
+
             divisions = (None,) * (len(rgs) + 1)
     else:
+        if infer_divisions is True:
+            raise ValueError(
+                'Unable to infer divisions for because no index column was discovered')
+
         divisions = (None,) * (len(rgs) + 1)
 
     if isinstance(divisions[0], np.datetime64):
@@ -381,7 +354,6 @@ def _write_partition_fastparquet(df, fs, path, filename, fmd, compression,
                                  partition_on):
     from fastparquet.writer import partition_on_columns, make_part_file
     import fastparquet
-    from distutils.version import LooseVersion
     # Fastparquet mutates this in a non-threadsafe manner. For now we just copy
     # it before forwarding to fastparquet.
     fmd = copy.copy(fmd)
@@ -506,7 +478,7 @@ def _write_metadata(writes, filenames, fmd, path, fs, sep):
 # PyArrow interface
 
 def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
-                  categories=None, index=None):
+                  categories=None, index=None, infer_divisions=None):
     from ...bytes.core import get_pyarrow_filesystem
     import pyarrow.parquet as pq
     import pyarrow as pa
@@ -545,7 +517,7 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
         storage_name_mapping = {k: k for k in column_names}
         column_index_names = [None]
 
-    if pa.__version__ < distutils.version.LooseVersion('0.8.0'):
+    if pa.__version__ < LooseVersion('0.8.0'):
         # the pyarrow 0.7.0 *reader* expects the storage names for index names
         # that are None.
         if any(x is None for x in index_names):
@@ -554,15 +526,53 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
             index_names = [name_storage_mapping.get(name, name)
                            for name in index_names]
 
-    column_names, index_names, out_type = _normalize_index_columns(columns, column_names,
-                                                                   index, index_names)
+    column_names, index_names, out_type = _normalize_index_columns(
+        columns, column_names, index, index_names)
 
     all_columns = index_names + column_names
 
+    # Find non-empty pieces
+    non_empty_pieces = []
+    # Determine valid pieces
+    _open = lambda fn: pq.ParquetFile(fs.open(fn, mode='rb'))
+    for piece in dataset.pieces:
+        pf = piece.get_metadata(_open)
+        # non_empty_pieces.append(piece)
+        if pf.num_row_groups > 0:
+            non_empty_pieces.append(piece)
+
+    # Sort pieces naturally
+    # If a single input path resulted in multiple dataset pieces, then sort
+    # the pieces naturally. If multiple paths were supplied then we leave
+    # the order of the resulting pieces unmodified
+    if len(paths) == 1 and len(dataset.pieces) > 1:
+        non_empty_pieces = sorted(
+            non_empty_pieces, key=lambda piece: natural_sort_key(piece.path))
+
+    # Determine divisions
+    if len(index_names) == 1:
+
+        # Look up storage name of the single index column
+        divisions_names = [storage_name for storage_name, name
+                           in storage_name_mapping.items()
+                           if index_names[0] == name]
+
+        if divisions_names:
+            divisions_name = divisions_names[0]
+        else:
+            divisions_name = None
+    else:
+        divisions_name = None
+
+    divisions = _get_pyarrow_divisions(non_empty_pieces, divisions_name,
+                                       schema, infer_divisions)
+
+    # Build task
     dtypes = _get_pyarrow_dtypes(schema, categories)
     dtypes = {storage_name_mapping.get(k, k): v for k, v in dtypes.items()}
 
-    meta = _meta_from_dtypes(all_columns, dtypes, index_names, column_index_names)
+    meta = _meta_from_dtypes(all_columns, dtypes, index_names,
+                             column_index_names)
     meta = clear_known_categories(meta, cols=categories)
 
     if out_type == Series:
@@ -571,8 +581,7 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
 
     task_name = 'read-parquet-' + tokenize(fs_token, paths, all_columns)
 
-    if dataset.pieces:
-        divisions = (None,) * (len(dataset.pieces) + 1)
+    if non_empty_pieces:
         task_plan = {
             (task_name, i): (_read_pyarrow_parquet_piece,
                              fs,
@@ -582,28 +591,139 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
                              out_type == Series,
                              dataset.partitions,
                              categories)
-            for i, piece in enumerate(dataset.pieces)
+            for i, piece in enumerate(non_empty_pieces)
         }
     else:
         meta = strip_unknown_categories(meta)
-        divisions = (None, None)
         task_plan = {(task_name, 0): meta}
 
     return out_type(task_plan, task_name, meta, divisions)
 
 
-def _get_pyarrow_dtypes(schema, categories):
-    dtypes = {}
-    for i in range(len(schema)):
-        field = schema[i]
-        numpy_dtype = field.type.to_pandas_dtype()
-        dtypes[field.name] = numpy_dtype
+def _to_ns(val, unit):
+    """
+    Convert an input time in the specified units to nanoseconds
 
-    if categories:
-        for cat in categories:
-            dtypes[cat] = "category"
+    Parameters
+    ----------
+    val: int
+        Input time value
+    unit : str
+        Time units of `val`.
+        One of 's', 'ms', 'us', 'ns'
 
-    return dtypes
+    Returns
+    -------
+    int
+        Time val in nanoseconds
+    """
+    factors = {'s': int(1e9), 'ms': int(1e6), 'us': int(1e3), 'ns': 1}
+    try:
+        factor = factors.get(unit)
+    except KeyError:
+        raise ValueError("Unsupported time unit '{unit}'".format(unit=unit))
+
+    return val * factor
+
+
+def _get_pyarrow_divisions(pa_pieces, divisions_name, pa_schema, infer_divisions):
+    """
+    Compute DataFrame divisions from a list of pyarrow dataset pieces
+
+    Parameters
+    ----------
+    pa_pieces : list[pyarrow.parquet.ParquetDatasetPiece]
+        List of dataset pieces. Each piece corresponds to a single partition in the eventual dask DataFrame
+    divisions_name : str|None
+        The name of the column to compute divisions for
+    pa_schema : pyarrow.lib.Schema
+        The pyarrow schema for the dataset
+    infer_divisions : bool or None
+        If True divisions must be inferred (otherwise an exception is raised). If False or None divisions are not
+        inferred
+    Returns
+    -------
+    list
+    """
+    # Local imports
+    import pyarrow as pa
+    import pyarrow.parquet as pq
+
+    if infer_divisions is True and pa.__version__ < LooseVersion('0.9.0'):
+        raise NotImplementedError('infer_divisions=True requires pyarrow >=0.9.0')
+
+    # Check whether divisions_name is in the schema
+    # Note: get_field_index returns -1 if not found, but it does not accept None
+    if infer_divisions is True:
+        divisions_name_in_schema = divisions_name is not None and pa_schema.get_field_index(divisions_name) >= 0
+
+        if divisions_name_in_schema is False and infer_divisions is True:
+            raise ValueError(
+                'Unable to infer divisions for because no index column was discovered')
+    else:
+        divisions_name_in_schema = None
+
+    if pa_pieces and divisions_name_in_schema:
+        # We have pieces and a valid division column.
+        # Compute min/max for column in each row group
+        min_maxs = []
+        last_max = None
+
+        # Initialize index of divisions column within the row groups.
+        # To be computed during while processing the first piece below
+        for piece in pa_pieces:
+            pf = piece.get_metadata(pq.ParquetFile)
+            rg = pf.row_group(0)
+
+            # Compute division column index
+            rg_paths = [rg.column(i).path_in_schema for i in range(rg.num_columns)]
+            try:
+                divisions_col_index = rg_paths.index(divisions_name)
+            except ValueError:
+                # Divisions not valid
+                min_maxs = None
+                break
+
+            col_meta = rg.column(divisions_col_index)
+            stats = col_meta.statistics
+            if stats.has_min_max and (last_max is None or last_max < stats.min):
+                min_maxs.append((stats.min, stats.max))
+                last_max = stats.max
+            else:
+                # Divisions not valid
+                min_maxs = None
+                break
+
+        if min_maxs:
+            # We have min/max pairs
+            divisions = [mn for mn, mx in min_maxs] + [min_maxs[-1][1]]
+
+            # Handle conversion to pandas timestamp divisions
+            index_field = pa_schema.field_by_name(divisions_name)
+            if isinstance(index_field.type, pa.TimestampType):
+                time_unit = index_field.type.unit
+                divisions_ns = [_to_ns(d, time_unit) for d in
+                                divisions]
+                divisions = [pd.Timestamp(ns) for ns in divisions_ns]
+
+            # Handle encoding of bytes string
+            if index_field.type == pa.string():
+                # Parquet strings are always encoded as utf-8
+                encoding = 'utf-8'
+                divisions = [d.decode(encoding).strip() for d in divisions]
+
+        else:
+            if infer_divisions is True:
+                raise ValueError(
+                    ("Unable to infer divisions for index of '{index_name}' because it is not known to be "
+                     "sorted across partitions").format(index_name=divisions_name_in_schema))
+
+            divisions = (None,) * (len(pa_pieces) + 1)
+    elif pa_pieces:
+        divisions = (None,) * (len(pa_pieces) + 1)
+    else:
+        divisions = (None, None)
+    return divisions
 
 
 def _read_pyarrow_parquet_piece(fs, piece, columns, index_cols, is_series,
@@ -616,7 +736,7 @@ def _read_pyarrow_parquet_piece(fs, piece, columns, index_cols, is_series,
                            use_pandas_metadata=True,
                            file=f)
 
-    if pa.__version__ < distutils.version.LooseVersion('0.9.0'):
+    if pa.__version__ < LooseVersion('0.9.0'):
         df = table.to_pandas()
         for cat in categories:
             df[cat] = df[cat].astype('category')
@@ -693,6 +813,7 @@ def _write_partition_pyarrow(df, path, fs, filename, write_index,
 
     if partition_on:
         parquet.write_to_dataset(t, path, partition_cols=partition_on,
+                                 preserve_index=write_index,
                                  filesystem=fs, **kwargs)
     else:
         with fs.open(filename, 'wb') as fil:
@@ -761,7 +882,7 @@ def get_engine(engine):
 
 
 def read_parquet(path, columns=None, filters=None, categories=None, index=None,
-                 storage_options=None, engine='auto'):
+                 storage_options=None, engine='auto', infer_divisions=None):
     """
     Read ParquetFile into a Dask DataFrame
 
@@ -802,10 +923,17 @@ def read_parquet(path, columns=None, filters=None, categories=None, index=None,
     engine : {'auto', 'fastparquet', 'pyarrow'}, default 'auto'
         Parquet reader library to use. If only one library is installed, it
         will use that one; if both, it will use 'fastparquet'
+    infer_divisions : bool or None (default).
+        By default, divisions are inferred if the read `engine` supports
+        doing so efficiently and the `index` of the underlying dataset is
+        sorted across the individual parquet files. Set to ``True`` to
+        force divisions to be inferred in all cases. Note that this may
+        require reading metadata from each file in the dataset, which may
+        be expensive. Set to ``False`` to never infer divisions.
 
     Examples
     --------
-    >>> df = read_parquet('s3://bucket/my-parquet-data')  # doctest: +SKIP
+    >>> df = dd.read_parquet('s3://bucket/my-parquet-data')  # doctest: +SKIP
 
     See Also
     --------
@@ -816,8 +944,13 @@ def read_parquet(path, columns=None, filters=None, categories=None, index=None,
     fs, fs_token, paths = get_fs_token_paths(path, mode='rb',
                                              storage_options=storage_options)
 
+    if isinstance(path, string_types) and len(paths) > 1:
+        # Sort paths naturally if multiple paths resulted from a single
+        # specification (by '*' globbing)
+        paths = sorted(paths, key=natural_sort_key)
+
     return read(fs, fs_token, paths, columns=columns, filters=filters,
-                categories=categories, index=index)
+                categories=categories, index=index, infer_divisions=infer_divisions)
 
 
 def to_parquet(df, path, engine='auto', compression='default', write_index=None,
@@ -887,6 +1020,8 @@ def to_parquet(df, path, engine='auto', compression='default', write_index=None,
 
     if compression != 'default':
         kwargs['compression'] = compression
+    elif 'snappy' in compress:
+        kwargs['compression'] = 'snappy'
 
     write = get_engine(engine)['write']
 
