@@ -270,6 +270,20 @@ def test_read_csv_files(dd_read, pd_read, files):
         assert_eq(df, expected2, check_dtype=False)
 
 
+@pytest.mark.parametrize('dd_read,pd_read,files',
+                         [(dd.read_csv, pd.read_csv, csv_files),
+                          (dd.read_table, pd.read_table, tsv_files)])
+def test_read_csv_files_list(dd_read, pd_read, files):
+    with filetexts(files, mode='b'):
+        subset = sorted(files)[:2]  # Just first 2
+        sol = pd.concat([pd_read(BytesIO(files[k])) for k in subset])
+        res = dd_read(subset)
+        assert_eq(res, sol, check_dtype=False)
+
+        with pytest.raises(ValueError):
+            dd_read([])
+
+
 # After this point, we test just using read_csv, as all functionality
 # for both is implemented using the same code.
 
@@ -384,6 +398,62 @@ def test_categorical_dtypes():
                 ['apple', 'banana', 'orange', 'pear'])
 
 
+@pytest.mark.skipif(PANDAS_VERSION < '0.21.0',
+                    reason="Uses CategoricalDtype")
+def test_categorical_known():
+    text1 = normalize_text("""
+    A,B
+    a,a
+    b,b
+    a,a
+    """)
+    text2 = normalize_text("""
+    A,B
+    a,a
+    b,b
+    c,c
+    """)
+    dtype = pd.api.types.CategoricalDtype(['a', 'b', 'c'])
+    with filetexts({"foo.1.csv": text1, "foo.2.csv": text2}):
+        result = dd.read_csv("foo.*.csv", dtype={"A": 'category',
+                                                 "B": 'category'})
+        assert result.A.cat.known is False
+        assert result.B.cat.known is False
+        expected = pd.DataFrame({
+            "A": pd.Categorical(['a', 'b', 'a', 'a', 'b', 'c'],
+                                categories=dtype.categories),
+            "B": pd.Categorical(['a', 'b', 'a', 'a', 'b', 'c'],
+                                categories=dtype.categories)},
+                                index=[0, 1, 2, 0, 1, 2])
+        assert_eq(result, expected)
+
+        # Specify a dtype
+        result = dd.read_csv("foo.*.csv", dtype={'A': dtype, 'B': 'category'})
+        assert result.A.cat.known is True
+        assert result.B.cat.known is False
+        tm.assert_index_equal(result.A.cat.categories, dtype.categories)
+        assert result.A.cat.ordered is False
+        assert_eq(result, expected)
+
+        # ordered
+        dtype = pd.api.types.CategoricalDtype(['a', 'b', 'c'], ordered=True)
+        result = dd.read_csv("foo.*.csv", dtype={'A': dtype, 'B': 'category'})
+        expected['A'] = expected['A'].cat.as_ordered()
+        assert result.A.cat.known is True
+        assert result.B.cat.known is False
+        assert result.A.cat.ordered is True
+
+        assert_eq(result, expected)
+
+        # Specify "unknown" categories
+        result = dd.read_csv("foo.*.csv",
+                             dtype=pd.api.types.CategoricalDtype())
+        assert result.A.cat.known is False
+
+        result = dd.read_csv("foo.*.csv", dtype="category")
+        assert result.A.cat.known is False
+
+
 @pytest.mark.slow
 def test_compression_multiple_files():
     with tmpdir() as tdir:
@@ -870,6 +940,20 @@ def test_error_if_sample_is_too_small():
                   pd.read_csv(fn, header=None, skiprows=3))
 
 
+def test_read_csv_names_not_none():
+    text = ('Alice,100\n'
+            'Bob,-200\n'
+            'Charlie,300\n'
+            'Dennis,400\n'
+            'Edith,-500\n'
+            'Frank,600\n')
+    names = ['name', 'amount']
+    with filetext(text) as fn:
+        ddf = dd.read_csv(fn, names=names, blocksize=16)
+        df = pd.read_csv(fn, names=names)
+        assert_eq(df, ddf, check_index=False)
+
+
 ############
 #  to_csv  #
 ############
